;******************************************************************************
; File Name:	 ch6_e1.asm
; Target System: C240x Evaluation Board
; Description: 	This program moves the stepper motor by 180 deg. in 5 seconds,
;				given the motor has a stepping angle of 1.8 deg or 200 step/revolution
;	       		T1 - T4 -> IOPB0-3 (Connector P1 Pin 5, 6, 7, 8)
;	       		TIMER 1 PERIOD INTERRUPT FOR 50MS I.E. 46875
;
; Date:			01/09/2004
; Author:		Min Dai
;******************************************************************************

;******************************************************************************
;					  SYSTEM OPTIONS
;******************************************************************************
real_time	.set	0      		; 1 for real time mode, otherwise set 0
T1PERIOD	.set	46875		; for 50ms, T1PR=30000000/32/20=46875, 20Hz=1/50ms
								; given 30MHz 2407 DSP, T1 prescaler=32
;T1PERIOD	.set	125000

;******************************************************************************

;-----------------------------------------------------------------------------
; External references
;-----------------------------------------------------------------------------
		.include 	"f2407.h"
;		.global 	MON_RT_CNFG

		.ref		SYS_INIT

;-----------------------------------------------------------------------------
; Local Variable Declarations
;-----------------------------------------------------------------------------
		.def		GPR0				;General purpose register.
        
		.bss		GPR0,1				;General purpose register. 
		.bss		STEPCTR,1			;variable for step counter
		.bss		STATECTR,1			;variable for state counter
		.bss		STATEVAL,1			;variable for state value
		
;==============================================================================
; V E C T O R    T A B L E    ( including RT monitor traps )
;==============================================================================
;        .include 	"c200mnrt.i" 		; Include conditional assembly options.

		.global		_c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;==============================================================================
; M A I N   C O D E  - starts here
;==============================================================================
		.text
_c_int0:
		CALL		SYS_INIT                ;DSP initialization

;----------------------------------------------------------
; Initialise the Real time monitor
;----------------------------------------------------------

		POINT_PG0

;---Real Time option---------------
	.if (real_time)
		CALL		MON_RT_CNFG				;For Real-Time
	.endif
;----------------------------------

;----------------------------------------------------------
; System Interrupt Init.
;----------------------------------------------------------

;---Real Time option --------------------------------------------------
	.if (real_time)
		SPLK	#0000000001000010b,IMR	;En Int lvl 2 & 7 for T1 and RT  
		        ;|||||||||!||||||
			    ;5432109876543210
	.endif


	.if (real_time != 1)
		SPLK	#0000000000000010b,IMR	;Enable Int 2 only for T1
			    ;||||!!!!||||!!!!		
			    ;5432109876543210
	.endif

		SPLK	#0FFFFh, IFR			;Clear any pending Ints
;		EINT							;Enable global Ints
;-----------------------------------------------------------------------
;-------------------------------------------------------------------------
;		RESET SECTION - BEGINS
;-------------------------------------------------------------------------
        LDP     #DP_EVA                 ;set data page
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB        ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC        ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA        ;enabled desired EVA group A interrupts
        SPLK    #00000h, EVAIMRB        ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC        ;enabled desired EVA group C interrupts

        LDP		#DP_EVA                 ;set data page
		SPLK   	#0, GPTCONA
		SPLK   	#0, T1CON
		SPLK   	#0, T2CON

		SPLK   	#0, COMCONA
		SPLK   	#0, ACTRA
		SPLK   	#0, DBTCONA
		SPLK   	#0, CAPCONA

;-------------------------------------------------------------------------
;		RESET SECTION - ENDS
;-------------------------------------------------------------------------
		LDP		#STATECTR
		SPLK	#0, STATECTR	;Initialize state counter
		SPLK	#0, STEPCTR		;Initialize step counter

		LDP		#DP_PF2
		SPLK	#0000h, MCRA	;Set pins to be I/O
		SPLK	#0F0Fh,	PBDATDIR;Select IOPB0-3 as outputs

		LDP    	#DP_EVA
		SPLK	#0055h, GPTCONA
		SPLK	#T1PERIOD, T1PR	;Set timer 1 period 50ms
		SPLK	#0, T1CNT
		SPLK	#0, T2CNT
 
		SPLK	#150Ah, T1CON	;Set PRESCALER=32
		
		SPLK   	#0080h, EVAIMRA
		
		SBIT1	T1CON, 	0000000001000000b	;Enable timer 1 by setting
					;	||||||||||||||||     Bit6 of T1CON to 1
					;	5432109876543210

		CLRC	INTM			;Enable interrupts
END		B		END


;====================================================================
; I S R - GISR2
; Description:	Determines the current state adn outputs the
; 		appropriate sequence to the transistors every
;		50ms. It also keeps track of the number of steps
;		and stops the motor when it has covered 180 degrees.
; Modifies:	STATEVAL, STATECTR, STEPCTR, PBDATDIR, EVIMRA
;====================================================================
GISR2
;Context save regs
		MAR		*,AR1	;AR1 is stack pointer
		MAR		*+      ;skip one position
		SST		#1, *+  ;save ST1
		SST   	#0, *+  ;save ST0
		SACH	*+      ;save ACC high
		SACL	*		;save ACC low

		LDP		#DP_EVA
		SPLK	#0FFFFh,EVAIFRA  	; Clear all Group A interrupt flags

;=========================================================
;Start main section of ISR
;=========================================================

		LDP		#STEPCTR
		LACC	STEPCTR
		ADD		#1
		SACL	STEPCTR			;Increment step counter
		SUB		#101
		BCND	NXT1,NEQ		;If number of steps < 100
								;goto NXT1
		LDP		#DP_PF2			;Load DP for I/O registers
		SPLK	#0F0Fh,PBDATDIR ;Turn off all transistors,
								;stop the motor if step=100
		LDP		#DP_EVA			;Load DP for event manager
		SPLK   	#0000h, EVAIMRA	;Disable timer 1 period interrupt
		B		FIN
NXT1	LDP		#STATECTR
		LACC	#STATE
		ADD		STATECTR		;ACC=address of current state
		TBLR	STATEVAL		;Move the state to a vairable
		LACC	STATEVAL		;Load the state
		LDP		#DP_PF2
		SACL	PBDATDIR		;Output the current state
		
		LDP		#STATECTR
		LACC	STATECTR
		SUB		#3
		BCND	NXT, NEQ		;If state counter < 3, jump to NXT
		SPLK	#0,STATECTR		;Reset state counter
		B		FIN
NXT		LACC	STATECTR
		ADD		#1
		SACL	STATECTR	;Increment state counter

FIN		
;=========================================================
;End main section of ISR
;=========================================================
;Context restore regs
		POINT_PG0

		MAR		*, AR1    	;make stack pointer active
		LACL	*-			;Restore Acc low
		ADDH	*-			;Restore Acc high
		LST		#0, *-      ;load ST0
		LST     #1, *-  	;load ST1
		EINT				;Enable all interrupts; clear the INTM to 0.
		RET



;==============================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:
;
; Last Update:	16-06-95
;==============================================================================
PHANTOM B	PHANTOM
GISR1 	RET
;GISR2 	RET
GISR3 	RET
GISR4 	RET
GISR5 	RET
GISR6	RET

STATE	.word	0F0Dh	     	;Position 2
        .word   0F0Bh         	;Position 1
        .word  	0F0Eh         	;Position 4
        .word   0F07h    		;Position 3

